@@ -15,7 +15,6 @@ from twisted.python import usage
 from twisted.internet import reactor
 
 # Standard library imports
-import string
 import sys
 from io import BytesIO
 
@@ -36,7 +35,7 @@ def success(response):
     if response is None:
         print(None)
     else:
-        print(string.join(response, '\n'))
+        print("\n".join(response))
     print('---')
 
 
@@ -47,9 +46,9 @@ def fail(error):
 def showFiles(result, fileListProtocol):
     print('Processed file listing:')
     for file in fileListProtocol.files:
-        print('    %s: %d bytes, %s' \
-              % (file['filename'], file['size'], file['date']))
-    print('Total: %d files' % (len(fileListProtocol.files)))
+        print('    {}: {} bytes, {}'.format(
+              file['filename'], file['size'], file['date']))
+    print('Total: {} files'.format(len(fileListProtocol.files)))
 
 def showBuffer(result, bufferProtocol):
     print('Got data:')
