@@ -6,6 +6,7 @@
 """
 An example client. Run simpleserv.py first before running this.
 """
+from __future__ import print_function
 
 from twisted.internet import reactor, protocol
 
@@ -16,25 +17,25 @@ class EchoClient(protocol.Protocol):
     """Once connected, send a message, then print the result."""
     
     def connectionMade(self):
-        self.transport.write("hello, world!")
+        self.transport.write(b"hello, world!")
     
     def dataReceived(self, data):
         "As soon as any data is received, write it back."
-        print "Server said:", data
+        print("Server said:", data)
         self.transport.loseConnection()
     
     def connectionLost(self, reason):
-        print "connection lost"
+        print("connection lost")
 
 class EchoFactory(protocol.ClientFactory):
     protocol = EchoClient
 
     def clientConnectionFailed(self, connector, reason):
-        print "Connection failed - goodbye!"
+        print("Connection failed - goodbye!")
         reactor.stop()
     
     def clientConnectionLost(self, connector, reason):
-        print "Connection lost - goodbye!"
+        print("Connection lost - goodbye!")
         reactor.stop()
 
 
