@@ -8,48 +8,42 @@ Tests for L{twisted.conch.scripts.cftp}.
 
 import locale
 import time, sys, os, operator, getpass, struct
-from StringIO import StringIO
+from io import BytesIO
 
+from twisted.python.filepath import FilePath
+from twisted.python.reflect import requireModule
 from zope.interface import implementer
-try:
-    import pyasn1
-except ImportError:
-    pyasn1 = None
-try:
-    import cryptography
-except ImportError:
-    cryptography = None
+
+pyasn1 = requireModule('pyasn1')
+cryptography = requireModule('cryptography')
+unix = requireModule('twisted.conch.unix')
 
 _reason = None
 if cryptography and pyasn1:
     try:
-        from twisted.conch import unix
         from twisted.conch.scripts import cftp
         from twisted.conch.scripts.cftp import SSHSession
+        from twisted.conch.ssh import filetransfer
         from twisted.conch.test.test_filetransfer import FileTransferForTestAvatar
-    except ImportError as e:
-        unix = None
-        _reason = str(e)
-        del e
-else:
-    unix = None
+        from twisted.conch.test import test_ssh, test_conch
+        from twisted.conch.test.test_conch import FakeStdio
+    except ImportError:
+        pass
 
-from twisted.python.fakepwd import UserDatabase
-from twisted.trial.unittest import TestCase
+from twisted.conch import ls
+from twisted.conch.interfaces import ISFTPFile
+from twisted.conch.test.test_filetransfer import SFTPTestBase
+from twisted.conch.test.test_filetransfer import FileTransferTestAvatar
 from twisted.cred import portal
 from twisted.internet import reactor, protocol, interfaces, defer, error
 from twisted.internet.utils import getProcessOutputAndValue, getProcessValue
 from twisted.python import log
-from twisted.conch import ls
-from twisted.conch.interfaces import ISFTPFile
-from twisted.conch.ssh import filetransfer
+from twisted.python.compat import _PY3, unicode
+from twisted.python.fakepwd import UserDatabase
 from twisted.test.proto_helpers import StringTransport
 from twisted.internet.task import Clock
+from twisted.trial.unittest import TestCase
 
-from twisted.conch.test import test_ssh, test_conch
-from twisted.conch.test.test_filetransfer import SFTPTestBase
-from twisted.conch.test.test_filetransfer import FileTransferTestAvatar
-from twisted.conch.test.test_conch import FakeStdio
 
 
 
@@ -78,6 +72,7 @@ class ListingTests(TestCase):
     if getattr(time, 'tzset', None) is None:
         skip = "Cannot test timestamp formatting code without time.tzset"
 
+
     def setUp(self):
         """
         Patch the L{ls} module's time function so the results of L{lsLine} are
@@ -309,7 +304,7 @@ class InMemorySFTPClient(object):
 
 
 @implementer(ISFTPFile)
-class InMemoryRemoteFile(StringIO):
+class InMemoryRemoteFile(BytesIO):
     """
     An L{ISFTPFile} which handles all data in memory.
     """
@@ -320,7 +315,7 @@ class InMemoryRemoteFile(StringIO):
         @type name: L{str}
         """
         self.name = name
-        StringIO.__init__(self)
+        BytesIO.__init__(self)
 
 
     def writeChunk(self, start, data):
@@ -338,8 +333,7 @@ class InMemoryRemoteFile(StringIO):
 
         Keeps data after file was closed to help with testing.
         """
-        if not self.closed:
-            self.closed = True
+        self._closed = True
 
 
     def getvalue(self):
@@ -348,10 +342,7 @@ class InMemoryRemoteFile(StringIO):
 
         Allow reading data event when file is closed.
         """
-        if self.buflist:
-            self.buf += ''.join(self.buflist)
-            self.buflist = []
-        return self.buf
+        return BytesIO.getvalue(self)
 
 
 
@@ -388,8 +379,8 @@ class StdioClientTests(TestCase):
             getpass.getuser(), 'secret', os.getuid(), 1234, 'foo', 'bar',
             sys.executable)
 
-        d = self.client._dispatchCommand("exec print 1 + 2")
-        d.addCallback(self.assertEqual, "3\n")
+        d = self.client._dispatchCommand("exec print(1 + 2)")
+        d.addCallback(self.assertEqual, b"3\n")
         return d
 
 
@@ -402,7 +393,7 @@ class StdioClientTests(TestCase):
             getpass.getuser(), 'secret', os.getuid(), 1234, 'foo', 'bar', '')
 
         d = self.client._dispatchCommand("exec echo hello")
-        d.addCallback(self.assertEqual, "hello\n")
+        d.addCallback(self.assertEqual, b"hello\n")
         return d
 
 
@@ -415,7 +406,7 @@ class StdioClientTests(TestCase):
             '/bin/sh')
 
         d = self.client._dispatchCommand("!echo hello")
-        d.addCallback(self.assertEqual, "hello\n")
+        d.addCallback(self.assertEqual, b"hello\n")
         return d
 
 
@@ -450,8 +441,8 @@ class StdioClientTests(TestCase):
         # need to test the console padding.
         self.setKnownConsoleSize(10, 34)
         clock = self.client.reactor = Clock()
-        wrapped = StringIO("x")
-        wrapped.name = "sample"
+        wrapped = BytesIO(b"x")
+        wrapped.name = b"sample"
         wrapper = cftp.FileWrapper(wrapped)
         wrapper.size = 1024 * 10
         startTime = clock.seconds()
@@ -460,8 +451,11 @@ class StdioClientTests(TestCase):
 
         self.client._printProgressBar(wrapper, startTime)
 
-        self.assertEqual(self.client.transport.value(),
-                          "\rsample 40% 4.0kB 2.0kBps 00:03 ")
+        if _PY3:
+            result = b"\rb'sample' 40% 4.0kB 2.0kBps 00:03 "
+        else:
+            result = "\rsample 40% 4.0kB 2.0kBps 00:03 "
+        self.assertEqual(self.client.transport.value(), result)
 
 
     def test_printProgressBarNoProgress(self):
@@ -472,15 +466,18 @@ class StdioClientTests(TestCase):
         """
         self.setKnownConsoleSize(10, 34)
         clock = self.client.reactor = Clock()
-        wrapped = StringIO("x")
-        wrapped.name = "sample"
+        wrapped = BytesIO(b"x")
+        wrapped.name = b"sample"
         wrapper = cftp.FileWrapper(wrapped)
         startTime = clock.seconds()
 
         self.client._printProgressBar(wrapper, startTime)
 
-        self.assertEqual(self.client.transport.value(),
-                          "\rsample  0% 0.0B 0.0Bps 00:00 ")
+        if _PY3:
+            result = b"\rb'sample'  0% 0.0B 0.0Bps 00:00 "
+        else:
+            result = "\rsample  0% 0.0B 0.0Bps 00:00 "
+        self.assertEqual(self.client.transport.value(), result)
 
 
     def test_printProgressBarEmptyFile(self):
@@ -488,16 +485,17 @@ class StdioClientTests(TestCase):
         Print the progress for empty files.
         """
         self.setKnownConsoleSize(10, 34)
-        wrapped = StringIO()
-        wrapped.name = 'empty-file'
+        wrapped = BytesIO()
+        wrapped.name = b'empty-file'
         wrapper = cftp.FileWrapper(wrapped)
 
         self.client._printProgressBar(wrapper, 0)
 
-        self.assertEqual(
-            '\rempty-file100% 0.0B 0.0Bps 00:00 ',
-            self.client.transport.value(),
-            )
+        if _PY3:
+            result = b"\rb'empty-file'100% 0.0B 0.0Bps 00:00 "
+        else:
+            result = "\rempty-file100% 0.0B 0.0Bps 00:00 "
+        self.assertEqual(result, self.client.transport.value())
 
 
     def test_getFilenameEmpty(self):
@@ -555,7 +553,7 @@ class StdioClientTests(TestCase):
         """
         if path is None:
             path = self.mktemp()
-        with open(path, 'w') as file:
+        with open(path, 'wb') as file:
             file.write(content)
         return path
 
@@ -570,7 +568,10 @@ class StdioClientTests(TestCase):
             in which put reposes are received
 
         """
-        output = self.client.transport.value().split('\n\r')
+        output = self.client.transport.value()
+        if _PY3:
+            output = output.decode("utf-8")
+        output = output.split('\n\r')
 
         expectedOutput = []
         actualOutput = []
@@ -626,7 +627,7 @@ class StdioClientTests(TestCase):
 
         The progress is updated while chunks are transferred.
         """
-        content = 'Test\r\nContent'
+        content = b'Test\r\nContent'
         localPath = self.makeFile(content=content)
         flags = (
             filetransfer.FXF_WRITE |
@@ -642,7 +643,7 @@ class StdioClientTests(TestCase):
         self.successResultOf(deferred)
 
         self.assertEqual(content, remoteFile.getvalue())
-        self.assertTrue(remoteFile.closed)
+        self.assertTrue(remoteFile._closed)
         self.checkPutMessage(
             [(localPath, remoteName,
                 ['76% 10.0B', '100% 13.0B', '100% 13.0B'])])
@@ -669,8 +670,8 @@ class StdioClientTests(TestCase):
         self.successResultOf(deferred)
 
         self.checkPutMessage([(localPath, remoteName, ['100% 0.0B'])])
-        self.assertTrue(remoteFile.closed)
-        self.assertEqual('', remoteFile.getvalue())
+        self.assertTrue(remoteFile._closed)
+        self.assertEqual(b'', remoteFile.getvalue())
 
 
     def test_cmd_PUTMultipleNoRemotePath(self):
@@ -700,10 +701,10 @@ class StdioClientTests(TestCase):
         deferred = self.client.cmd_PUT(os.path.join(parent, '*'))
         self.successResultOf(deferred)
 
-        self.assertTrue(firstRemoteFile.closed)
-        self.assertEqual('', firstRemoteFile.getvalue())
-        self.assertTrue(secondRemoteFile.closed)
-        self.assertEqual('', secondRemoteFile.getvalue())
+        self.assertTrue(firstRemoteFile._closed)
+        self.assertEqual(b'', firstRemoteFile.getvalue())
+        self.assertTrue(secondRemoteFile._closed)
+        self.assertEqual(b'', secondRemoteFile.getvalue())
         self.checkPutMessage([
             (first, firstRemotePath, ['100% 0.0B']),
             (second, secondRemotePath, ['100% 0.0B']),
@@ -742,10 +743,10 @@ class StdioClientTests(TestCase):
             '%s remote' % (os.path.join(parent, '*'),))
         self.successResultOf(deferred)
 
-        self.assertTrue(firstRemoteFile.closed)
-        self.assertEqual('', firstRemoteFile.getvalue())
-        self.assertTrue(secondRemoteFile.closed)
-        self.assertEqual('', secondRemoteFile.getvalue())
+        self.assertTrue(firstRemoteFile._closed)
+        self.assertEqual(b'', firstRemoteFile.getvalue())
+        self.assertTrue(secondRemoteFile._closed)
+        self.assertEqual(b'', secondRemoteFile.getvalue())
         self.checkPutMessage([
             (first, firstName, ['100% 0.0B']),
             (second, secondName, ['100% 0.0B']),
@@ -759,11 +760,13 @@ class FileTransferTestRealm:
     def __init__(self, testDir):
         self.testDir = testDir
 
+
     def requestAvatar(self, avatarID, mind, *interfaces):
         a = FileTransferTestAvatar(self.testDir)
         return interfaces[0], a, lambda: None
 
 
+
 class SFTPTestProcess(protocol.ProcessProtocol):
     """
     Protocol for testing cftp. Provides an interface between Python (where all
@@ -782,20 +785,22 @@ class SFTPTestProcess(protocol.ProcessProtocol):
         self._expectingCommand = None
         self._processEnded = False
 
+
     def clearBuffer(self):
         """
         Clear any buffered data received from stdout. Should be private.
         """
-        self.buffer = ''
+        self.buffer = b''
         self._linesReceived = []
-        self._lineBuffer = ''
+        self._lineBuffer = b''
+
 
     def outReceived(self, data):
         """
         Called by Twisted when the cftp client prints data to stdout.
         """
-        log.msg('got %s' % data)
-        lines = (self._lineBuffer + data).split('\n')
+        log.msg('got %r' % data)
+        lines = (self._lineBuffer + data).split(b'\n')
         self._lineBuffer = lines.pop(-1)
         self._linesReceived.extend(lines)
         # XXX - not strictly correct.
@@ -807,28 +812,32 @@ class SFTPTestProcess(protocol.ProcessProtocol):
         self.buffer += data
         self._checkForCommand()
 
+
     def _checkForCommand(self):
-        prompt = 'cftp> '
+        prompt = b'cftp> '
         if self._expectingCommand and self._lineBuffer == prompt:
-            buf = '\n'.join(self._linesReceived)
+            buf = b'\n'.join(self._linesReceived)
             if buf.startswith(prompt):
                 buf = buf[len(prompt):]
             self.clearBuffer()
             d, self._expectingCommand = self._expectingCommand, None
             d.callback(buf)
 
+
     def errReceived(self, data):
         """
         Called by Twisted when the cftp client prints data to stderr.
         """
         log.msg('err: %s' % data)
 
+
     def getBuffer(self):
         """
         Return the contents of the buffer of data received from stdout.
         """
         return self.buffer
 
+
     def runCommand(self, command):
         """
         Issue the given command via the cftp client. Return a C{Deferred} that
@@ -842,9 +851,12 @@ class SFTPTestProcess(protocol.ProcessProtocol):
         """
         self._expectingCommand = defer.Deferred()
         self.clearBuffer()
-        self.transport.write(command + '\n')
+        if isinstance(command, unicode):
+            command = command.encode("utf-8")
+        self.transport.write(command + b'\n')
         return self._expectingCommand
 
+
     def runScript(self, commands):
         """
         Run each command in sequence and return a Deferred that fires when all
@@ -860,6 +872,7 @@ class SFTPTestProcess(protocol.ProcessProtocol):
         dl = [sem.run(self.runCommand, command) for command in commands]
         return defer.gatherResults(dl)
 
+
     def killProcess(self):
         """
         Kill the process if it is still running.
@@ -875,6 +888,7 @@ class SFTPTestProcess(protocol.ProcessProtocol):
         self.transport.signalProcess('KILL')
         return self.onProcessEnd
 
+
     def processEnded(self, reason):
         """
         Called by Twisted when the cftp client process ends.
@@ -885,17 +899,19 @@ class SFTPTestProcess(protocol.ProcessProtocol):
             d.callback(None)
 
 
+
 class CFTPClientTestBase(SFTPTestBase):
     def setUp(self):
-        with open('dsa_test.pub','w') as f:
+        with open('dsa_test.pub', 'wb') as f:
             f.write(test_ssh.publicDSA_openssh)
-        with open('dsa_test','w') as f:
+        with open('dsa_test', 'wb') as f:
             f.write(test_ssh.privateDSA_openssh)
         os.chmod('dsa_test', 33152)
-        with open('kh_test','w') as f:
-            f.write('127.0.0.1 ' + test_ssh.publicRSA_openssh)
+        with open('kh_test', 'wb') as f:
+            f.write(b'127.0.0.1 ' + test_ssh.publicRSA_openssh)
         return SFTPTestBase.setUp(self)
 
+
     def startServer(self):
         realm = FileTransferTestRealm(self.testDir)
         p = portal.Portal(realm)
@@ -904,6 +920,7 @@ class CFTPClientTestBase(SFTPTestBase):
         fac.portal = p
         self.server = reactor.listenTCP(0, fac, interface="127.0.0.1")
 
+
     def stopServer(self):
         if not hasattr(self.server.factory, 'proto'):
             return self._cbStopServer(None)
@@ -913,9 +930,11 @@ class CFTPClientTestBase(SFTPTestBase):
         d.addCallback(self._cbStopServer)
         return d
 
+
     def _cbStopServer(self, ignored):
         return defer.maybeDeferred(self.server.stopListening)
 
+
     def tearDown(self):
         for f in ['dsa_test.pub', 'dsa_test', 'kh_test']:
             try:
@@ -955,21 +974,39 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         d.addCallback(lambda _: self.processProtocol.clearBuffer())
         env = os.environ.copy()
         env['PYTHONPATH'] = os.pathsep.join(sys.path)
-        reactor.spawnProcess(self.processProtocol, sys.executable, cmds,
-                             env=env)
+        encodedCmds = []
+        encodedEnv = {}
+        for cmd in cmds:
+            if isinstance(cmd, unicode):
+                cmd = cmd.encode("utf-8")
+            encodedCmds.append(cmd)
+        for var in env:
+            val = env[var]
+            if isinstance(var, unicode):
+                var = var.encode("utf-8")
+            if isinstance(val, unicode):
+                val = val.encode("utf-8")
+            encodedEnv[var] = val
+        log.msg(encodedCmds)
+        log.msg(encodedEnv)
+        reactor.spawnProcess(self.processProtocol, sys.executable, encodedCmds,
+                             env=encodedEnv)
         return d
 
+
     def tearDown(self):
         d = self.stopServer()
         d.addCallback(lambda _: self.processProtocol.killProcess())
         return d
 
+
     def _killProcess(self, ignored):
         try:
             self.processProtocol.transport.signalProcess('KILL')
         except error.ProcessExitedAlready:
             pass
 
+
     def runCommand(self, command):
         """
         Run the given command with the cftp client. Return a C{Deferred} that
@@ -978,6 +1015,7 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         """
         return self.processProtocol.runCommand(command)
 
+
     def runScript(self, *commands):
         """
         Run the given commands with the cftp client. Returns a C{Deferred}
@@ -986,6 +1024,7 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         """
         return self.processProtocol.runScript(commands)
 
+
     def testCdPwd(self):
         """
         Test that 'pwd' reports the current remote directory, that 'lpwd'
@@ -994,13 +1033,26 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         remote directory.
         """
         # XXX - not actually a unit test, see docstring.
-        homeDir = os.path.join(os.getcwd(), self.testDir)
+        homeDir = self.testDir
         d = self.runScript('pwd', 'lpwd', 'cd testDirectory', 'cd ..', 'pwd')
-        d.addCallback(lambda xs: xs[:3] + xs[4:])
+
+        def cmdOutput(output):
+            """
+            Callback function for handling command output.
+            """
+            cmds = []
+            for cmd in output:
+                if _PY3 and isinstance(cmd, bytes):
+                    cmd = cmd.decode("utf-8")
+                cmds.append(cmd)
+            return cmds[:3] + cmds[4:]
+
+        d.addCallback(cmdOutput)
         d.addCallback(self.assertEqual,
-                      [homeDir, os.getcwd(), '', homeDir])
+                      [homeDir.path, os.getcwd(), '', homeDir.path])
         return d
 
+
     def testChAttrs(self):
         """
         Check that 'ls -l' output includes the access permissions and that
@@ -1008,10 +1060,10 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         """
         def _check(results):
             self.flushLoggedErrors()
-            self.assertTrue(results[0].startswith('-rw-r--r--'))
-            self.assertEqual(results[1], '')
-            self.assertTrue(results[2].startswith('----------'), results[2])
-            self.assertEqual(results[3], '')
+            self.assertTrue(results[0].startswith(b'-rw-r--r--'))
+            self.assertEqual(results[1], b'')
+            self.assertTrue(results[2].startswith(b'----------'), results[2])
+            self.assertEqual(results[3], b'')
 
         d = self.runScript('ls -l testfile1', 'chmod 0 testfile1',
                            'ls -l testfile1', 'chmod 644 testfile1')
@@ -1025,17 +1077,17 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         listing directories and listing empty directories.
         """
         def _check(results):
-            self.assertEqual(results[0], ['testDirectory', 'testRemoveFile',
-                                          'testRenameFile', 'testfile1'])
-            self.assertEqual(results[1], ['testDirectory', 'testRemoveFile',
-                                          'testRenameFile', 'testfile1'])
-            self.assertEqual(results[2], ['testRemoveFile', 'testRenameFile'])
-            self.assertEqual(results[3], ['.testHiddenFile', 'testRemoveFile',
-                                          'testRenameFile'])
-            self.assertEqual(results[4], [''])
-        d = self.runScript('ls', 'ls ../' + os.path.basename(self.testDir),
+            self.assertEqual(results[0], [b'testDirectory', b'testRemoveFile',
+                                          b'testRenameFile', b'testfile1'])
+            self.assertEqual(results[1], [b'testDirectory', b'testRemoveFile',
+                                          b'testRenameFile', b'testfile1'])
+            self.assertEqual(results[2], [b'testRemoveFile', b'testRenameFile'])
+            self.assertEqual(results[3], [b'.testHiddenFile', b'testRemoveFile',
+                                          b'testRenameFile'])
+            self.assertEqual(results[4], [b''])
+        d = self.runScript('ls', 'ls ../' + self.testDir.basename(),
                            'ls *File', 'ls -a *File', 'ls -l testDirectory')
-        d.addCallback(lambda xs: [x.split('\n') for x in xs])
+        d.addCallback(lambda xs: [x.split(b'\n') for x in xs])
         return d.addCallback(_check)
 
 
@@ -1044,17 +1096,20 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         Check that running the '?' command returns help.
         """
         d = self.runCommand('?')
-        d.addCallback(self.assertEqual,
-                      cftp.StdioClient(None).cmd_HELP('').strip())
+
+        helpText = cftp.StdioClient(None).cmd_HELP('').strip()
+        if isinstance(helpText, unicode):
+            helpText = helpText.encode("utf-8")
+        d.addCallback(self.assertEqual, helpText)
         return d
 
+
     def assertFilesEqual(self, name1, name2, msg=None):
         """
         Assert that the files at C{name1} and C{name2} contain exactly the
         same data.
         """
-        with open(name1) as f1, open(name2) as f2:
-            self.assertEqual(f1.read(), f2.read(), msg)
+        self.assertEqual(name1.getContent(), name2.getContent(), msg)
 
 
     def testGet(self):
@@ -1064,19 +1119,21 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         the file.
         """
         # XXX - not actually a unit test
-        expectedOutput = ("Transferred %s/%s/testfile1 to %s/test file2"
-                          % (os.getcwd(), self.testDir, self.testDir))
+        expectedOutput = ("Transferred %s/testfile1 to %s/test file2"
+                          % (self.testDir.path, self.testDir.path))
+        if isinstance(expectedOutput, unicode):
+            expectedOutput = expectedOutput.encode("utf-8")
         def _checkGet(result):
             self.assertTrue(result.endswith(expectedOutput))
-            self.assertFilesEqual(self.testDir + '/testfile1',
-                                  self.testDir + '/test file2',
+            self.assertFilesEqual(self.testDir.child('testfile1'),
+                                  self.testDir.child('test file2'),
                                   "get failed")
             return self.runCommand('rm "test file2"')
 
-        d = self.runCommand('get testfile1 "%s/test file2"' % (self.testDir,))
+        d = self.runCommand('get testfile1 "%s/test file2"' % (self.testDir.path,))
         d.addCallback(_checkGet)
         d.addCallback(lambda _: self.assertFalse(
-            os.path.exists(self.testDir + '/test file2')))
+            self.testDir.child('test file2').exists()))
         return d
 
 
@@ -1085,11 +1142,11 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         Test that 'get' works correctly when given wildcard parameters.
         """
         def _check(ignored):
-            self.assertFilesEqual(self.testDir + '/testRemoveFile',
-                                  'testRemoveFile',
+            self.assertFilesEqual(self.testDir.child('testRemoveFile'),
+                                  FilePath('testRemoveFile'),
                                   'testRemoveFile get failed')
-            self.assertFilesEqual(self.testDir + '/testRenameFile',
-                                  'testRenameFile',
+            self.assertFilesEqual(self.testDir.child('testRenameFile'),
+                                  FilePath('testRenameFile'),
                                   'testRenameFile get failed')
 
         d = self.runCommand('get testR*')
@@ -1102,19 +1159,20 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         successfully removed. Also check the output of the put command.
         """
         # XXX - not actually a unit test
-        expectedOutput = ('Transferred %s/testfile1 to %s/%s/test"file2'
-                          % (self.testDir, os.getcwd(), self.testDir))
+        expectedOutput = (b'Transferred ' + self.testDir.asBytesMode().path +
+                          b'/testfile1 to ' + self.testDir.asBytesMode().path +
+                          b'/test"file2')
         def _checkPut(result):
-            self.assertFilesEqual(self.testDir + '/testfile1',
-                                  self.testDir + '/test"file2')
+            self.assertFilesEqual(self.testDir.child('testfile1'),
+                                  self.testDir.child('test"file2'))
             self.assertTrue(result.endswith(expectedOutput))
             return self.runCommand('rm "test\\"file2"')
 
         d = self.runCommand('put %s/testfile1 "test\\"file2"'
-                            % (self.testDir,))
+                            % (self.testDir.path,))
         d.addCallback(_checkPut)
         d.addCallback(lambda _: self.assertFalse(
-            os.path.exists(self.testDir + '/test"file2')))
+            self.testDir.child('test"file2').exists()))
         return d
 
 
@@ -1124,16 +1182,16 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         file.
         """
         # XXX - not actually a unit test
-        with open(os.path.join(self.testDir, 'shorterFile'), 'w') as f:
-            f.write("a")
-        with open(os.path.join(self.testDir, 'longerFile'), 'w') as f:
-            f.write("bb")
+        with self.testDir.child('shorterFile').open(mode='w') as f:
+            f.write(b"a")
+        with self.testDir.child('longerFile').open(mode='w') as f:
+            f.write(b"bb")
         def _checkPut(result):
-            self.assertFilesEqual(self.testDir + '/shorterFile',
-                                  self.testDir + '/longerFile')
+            self.assertFilesEqual(self.testDir.child('shorterFile'),
+                                  self.testDir.child('longerFile'))
 
         d = self.runCommand('put %s/shorterFile longerFile'
-                            % (self.testDir,))
+                            % (self.testDir.path,))
         d.addCallback(_checkPut)
         return d
 
@@ -1144,17 +1202,18 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         file and you use a wildcard to specify the files to upload.
         """
         # XXX - not actually a unit test
-        os.mkdir(os.path.join(self.testDir, 'dir'))
-        with open(os.path.join(self.testDir, 'dir', 'file'), 'w') as f:
-            f.write("a")
-        with open(os.path.join(self.testDir, 'file'), 'w') as f:
-            f.write("bb")
+        someDir = self.testDir.child('dir')
+        someDir.createDirectory()
+        with someDir.child('file').open(mode='w') as f:
+            f.write(b"a")
+        with self.testDir.child('file').open(mode='w') as f:
+            f.write(b"bb")
         def _checkPut(result):
-            self.assertFilesEqual(self.testDir + '/dir/file',
-                                  self.testDir + '/file')
+            self.assertFilesEqual(someDir.child('file'),
+                                  self.testDir.child('file'))
 
         d = self.runCommand('put %s/dir/*'
-                            % (self.testDir,))
+                            % (self.testDir.path,))
         d.addCallback(_checkPut)
         return d
 
@@ -1166,18 +1225,19 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         uploaded to the correct directory.
         """
         def check(results):
-            self.assertEqual(results[0], '')
-            self.assertEqual(results[2], '')
-            self.assertFilesEqual(self.testDir + '/testRemoveFile',
-                                  self.testDir + '/../testRemoveFile',
+            self.assertEqual(results[0], b'')
+            self.assertEqual(results[2], b'')
+
+            self.assertFilesEqual(self.testDir.child('testRemoveFile'),
+                                  self.testDir.parent().child('testRemoveFile'),
                                   'testRemoveFile get failed')
-            self.assertFilesEqual(self.testDir + '/testRenameFile',
-                                  self.testDir + '/../testRenameFile',
+            self.assertFilesEqual(self.testDir.child('testRenameFile'),
+                                  self.testDir.parent().child('testRenameFile'),
                                   'testRenameFile get failed')
 
         d = self.runScript('cd ..',
-                           'put %s/testR*' % (self.testDir,),
-                           'cd %s' % os.path.basename(self.testDir))
+                           'put %s/testR*' % (self.testDir.path,),
+                           'cd %s' % self.testDir.basename())
         d.addCallback(check)
         return d
 
@@ -1189,13 +1249,13 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         """
         def _check(results):
             self.flushLoggedErrors()
-            self.assertEqual(results[0], '')
-            self.assertTrue(results[1].startswith('l'), 'link failed')
+            self.assertEqual(results[0], b'')
+            self.assertTrue(results[1].startswith(b'l'), 'link failed')
             return self.runCommand('rm testLink')
 
         d = self.runScript('ln testLink testfile1', 'ls -l testLink')
         d.addCallback(_check)
-        d.addCallback(self.assertEqual, '')
+        d.addCallback(self.assertEqual, b'')
         return d
 
 
@@ -1204,14 +1264,14 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         Test that we can create and remove directories with the cftp client.
         """
         def _check(results):
-            self.assertEqual(results[0], '')
-            self.assertTrue(results[1].startswith('d'))
+            self.assertEqual(results[0], b'')
+            self.assertTrue(results[1].startswith(b'd'))
             return self.runCommand('rmdir testMakeDirectory')
 
         d = self.runScript('mkdir testMakeDirectory',
                            'ls -l testMakeDirector?')
         d.addCallback(_check)
-        d.addCallback(self.assertEqual, '')
+        d.addCallback(self.assertEqual, b'')
         return d
 
 
@@ -1221,9 +1281,9 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         appropriate error, and doesn't log an useless error server side.
         """
         def _check(results):
-            self.assertEqual(results[0], '')
+            self.assertEqual(results[0], b'')
             self.assertEqual(results[1],
-                              'remote error 11: mkdir failed')
+                              b'remote error 11: mkdir failed')
 
         d = self.runScript('mkdir testMakeDirectory',
                            'mkdir testMakeDirectory')
@@ -1237,10 +1297,10 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         cftp client. This test works because the 'remote' server is running
         out of a local directory.
         """
-        d = self.runCommand('lmkdir %s/testLocalDirectory' % (self.testDir,))
-        d.addCallback(self.assertEqual, '')
+        d = self.runCommand('lmkdir %s/testLocalDirectory' % (self.testDir.path,))
+        d.addCallback(self.assertEqual, b'')
         d.addCallback(lambda _: self.runCommand('rmdir testLocalDirectory'))
-        d.addCallback(self.assertEqual, '')
+        d.addCallback(self.assertEqual, b'')
         return d
 
 
@@ -1249,13 +1309,13 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         Test that we can rename a file.
         """
         def _check(results):
-            self.assertEqual(results[0], '')
-            self.assertEqual(results[1], 'testfile2')
+            self.assertEqual(results[0], b'')
+            self.assertEqual(results[1], b'testfile2')
             return self.runCommand('rename testfile2 testfile1')
 
         d = self.runScript('rename testfile1 testfile2', 'ls testfile?')
         d.addCallback(_check)
-        d.addCallback(self.assertEqual, '')
+        d.addCallback(self.assertEqual, b'')
         return d
 
 
@@ -1270,10 +1330,12 @@ class OurServerBatchFileTests(CFTPClientTestBase):
         CFTPClientTestBase.setUp(self)
         self.startServer()
 
+
     def tearDown(self):
         CFTPClientTestBase.tearDown(self)
         return self.stopServer()
 
+
     def _getBatchOutput(self, f):
         fn = self.mktemp()
         with open(fn, 'w') as fp:
@@ -1304,41 +1366,49 @@ class OurServerBatchFileTests(CFTPClientTestBase):
 
         return d
 
+
     def testBatchFile(self):
-        """Test whether batch file function of cftp ('cftp -b batchfile').
+        """
+        Test whether batch file function of cftp ('cftp -b batchfile').
         This works by treating the file as a list of commands to be run.
         """
         cmds = """pwd
 ls
 exit
 """
+
         def _cbCheckResult(res):
-            res = res.split('\n')
-            log.msg('RES %s' % str(res))
-            self.assertIn(self.testDir, res[1])
-            self.assertEqual(res[3:-2], ['testDirectory', 'testRemoveFile',
-                                             'testRenameFile', 'testfile1'])
+            res = res.split(b'\n')
+            log.msg('RES %s' % repr(res))
+            self.assertIn(self.testDir.asBytesMode().path, res[1])
+            self.assertEqual(res[3:-2], [b'testDirectory', b'testRemoveFile',
+                                             b'testRenameFile', b'testfile1'])
 
         d = self._getBatchOutput(cmds)
         d.addCallback(_cbCheckResult)
         return d
 
+
     def testError(self):
-        """Test that an error in the batch file stops running the batch.
+        """
+        Test that an error in the batch file stops running the batch.
         """
         cmds = """chown 0 missingFile
 pwd
 exit
 """
+
         def _cbCheckResult(res):
-            self.assertNotIn(self.testDir, res)
+            self.assertNotIn(self.testDir.asBytesMode().path, res)
 
         d = self._getBatchOutput(cmds)
         d.addCallback(_cbCheckResult)
         return d
 
+
     def testIgnoredError(self):
-        """Test that a minus sign '-' at the front of a line ignores
+        """
+        Test that a minus sign '-' at the front of a line ignores
         any errors.
         """
         cmds = """-chown 0 missingFile
@@ -1346,7 +1416,7 @@ pwd
 exit
 """
         def _cbCheckResult(res):
-            self.assertIn(self.testDir, res)
+            self.assertIn(self.testDir.asBytesMode().path, res)
 
         d = self._getBatchOutput(cmds)
         d.addCallback(_cbCheckResult)
@@ -1413,8 +1483,8 @@ class OurServerSftpClientTests(CFTPClientTestBase):
 
         def check(result):
             self.assertEqual(result[2], 0)
-            for i in ['testDirectory', 'testRemoveFile',
-                      'testRenameFile', 'testfile1']:
+            for i in [b'testDirectory', b'testRemoveFile',
+                      b'testRenameFile', b'testfile1']:
                 self.assertIn(i, result[0])
         d.addCallback(hasPAKT)
         d.addCallback(lambda args: getProcessOutputAndValue('sftp', args))
